function [g,info] = simplenormboundedrealgain(G,IQC,Fbasis,Fgrad,RateBounds, FlgPos)
% function [g,info] =  simplenormboundedgain(G,PI,Fbasis,Fgrad,RateBounds)
%
% Compute bound on gain of Fu(G,Delta) where G is an LTI system and Delta
% is an operator described by IQCs PI.
%
% Inputs
%  G: Linear parameter varying system.
%  IQC: Cell array containing the IQC factorization {Psi_i,M_i} for
%       the i^th multiplier for Delta. This function assumes Delta
%       is square m-by-m.
% Fbasis: Basis functions used in storage function Nbasis-by-1-by-nmod
% Fgrad: Gradient of the basis functions wrt. the parameters:
%        Nbasis-by-Nparameters-by-nmod
% RateBounds: Upper and lower bounds on the parameter rates: Nparameter-by-2
% FlgPos: 1 to enforce P>=0 

% Outputs
%  g: Bound on the gain of Fu(G,Delta)
%  info:  Structure with information regarding solution of dissipation
%         inequality. Structure includes the fields:
%    -P:  Symmetric matrix used to construct the storage function.
%    -lamopt: Coefficients for optimal conic combination of IQC multipliers
%

%% Dimensions
szG = size(G);
if numel(szG) == 2
    szG = [szG 1];
end
nmod = szG(3); % # of model in the G array.
nve = szG(1);  % # of outputs of G = nv + ne
nwd = szG(2);  % # of inputs of G = nw + nd

Niqc = size(IQC,2);    % # of IQCs
szPsi = size(IQC{1}{1},2);
nv = floor(szPsi/2);   % # of inputs to Delta
nw = nv;              % # of outputs of Delta (Delta assumed to be square)
nd = nwd-nw;   % # of disturbances
ne = nve-nv;   % # of errors

npar = size(RateBounds,1); % # of parameters
nbasis = size(Fbasis,1); % # of basis functions


% Check for non-rate bounded case
ratebndflg = true;
if (nbasis==1 && npar==0) 
    ratebndflg = false;
end

%% Compute hard factorization for each IQC multiplier
Psi = cell(Niqc,1);
M = cell(Niqc,1);
for i=1:Niqc
    Psi{i} = IQC{i}{1};
    M{i} = IQC{i}{2};
end

%%
% Construct extended system Pext including dynamics of plant G and
% all auxiliary systems Psi. The I/O relationship for Pext is:
%     [z1; ...; zNiqc; e] = Pext*[w; d]
% where zj is the output of the auxiliary system Psi for the j^th IQC.

% Stack all auxiliary systems:
%      [z1; ...; zNiqc] = AllPsi*[v; w]
% zidx is a cell array of the output indices for zi.
AllPsi = vertcat(Psi{:});
nz = size(AllPsi,1);

zptr = 0;
zidx = cell(Niqc,1);
for i=1:Niqc
    zidx{i} = zptr + (1:size(M{i},1)*nv); 
    zptr = zptr + size(M{i},1)*nv;
end

% Form mapping [z;e] = AllPsiI*[v;w;e] 
AllPsiI = blkdiag(AllPsi,eye(ne));

% Form mapping [v;w;e]=Gw*[w;d] 
GI = [G; eye(nwd)];
vweidx = [1:nv, (nv+ne)+(1:nw), nv+(1:ne)];
Gw = GI(vweidx,:);

% Form [z;e]=Pext*[w;d] from the product AllPsi*M
% Pext = ss( zeros(nz+ne,nwd,nmod) );
for i=1:nmod
    Gwi = Gw(:,:,i);
    Pext(:,:,i) = AllPsiI*Gwi;
end

% Partition state matrices of Pext based on inputs [w;d] and outputs [z;e]
[A,B,C,D] = ssdata(Pext);
nx = size(A,1);
Ce = C(end-ne+1:end,:,:);
De = D(end-ne+1:end,:,:);

%%
% Initialize LMI and define variables
setlmis([])
X = zeros(nbasis,1);
for i1 = 1:nbasis
    X(i1) = lmivar(1,[nx 1]);    
end


for i2 = 1:Niqc
    nM = size(M{i2},1);
    [lam(i2),nlam(i2),xlam{i2}] = lmivar(1,[nM/2 1]);
    [R(i2),ndec,xR{i2}] = lmivar(3,skewdec(nM/2,nlam(i2)));
end
[gsq,ndec] = lmivar(1,[1 1]);
gsqI = lmivar(3,blkdiag(zeros(nx+nw),ndec*eye(nd)));

%%
% Specify LMIs

% LMI for Dissipation Inequality - Loop through array of models
cnt = 1;
for k1 = 1:nmod
    Ak = A(:,:,k1);
    Bk = B(:,:,k1);
    Cek = Ce(:,:,k1);
    Dek = De(:,:,k1);
    
    if ratebndflg
        Fgk1 = Fgrad(:,:,k1);        
    else
        Fgk1 = [];        
    end
    Fbk1 = Fbasis(:,:,k1);
            
    % L2 Bound LMI in block 3-by-3 form with gamma
    for q = 1:2^npar  % number of variables appearing in X's basis fcns
        
        for ibasis=1:nbasis
            lmiterm([cnt 1 1 X(ibasis)],[eye(nx); zeros(nw+nd,nx)],[Ak Bk]*Fbk1(ibasis,1),'s');
        end
        
        %L2 performance term
        lmiterm([-cnt 1 1 gsqI],1,1);
        lmiterm([cnt 1 1 0],[Cek Dek]'*[Cek Dek]);
        for i1=1:Niqc
            % Grab i1^th IQC
%             Mi = M{i1};
            Cyi = C(zidx{i1},:,k1);
            Dyi = D(zidx{i1},:,k1);
%             Mtil = [Cyi Dyi]'*Mi*[Cyi Dyi];
%             lmiterm([cnt 1 1 lam(i1)],Mtil,1,'s');
%             
            [Mi(i1),~,xMi] = lmivar(3,[xlam{i1},xR{i1};xR{i1}',-xlam{i1}]);
            lmiterm([cnt 1 1 Mi(i1)],[Cyi Dyi]',[Cyi Dyi]);
            
        end        
        
        if ratebndflg
            rbvec = RateBounds(:,1);
            tmp = dec2bin(q-1,npar);
            idx = find(tmp=='1');
            rbvec(idx) = RateBounds(idx,2);              
            for k2=1:nbasis
                for k3=1:npar
                    lmiterm([-cnt 1 1 X(k2)],[eye(nx); zeros(nwd,nx)],...
                            0.5*rbvec(k3)*Fgk1(k2,k3)*...
                            [eye(nx) zeros(nx,nwd)],'s');
                end
            end
        end
        cnt = cnt+1;
        
    end
    % LMI to enforce X>=0
    if FlgPos
        if ratebndflg || k1 == 1
            for n1=1:nbasis
                lmiterm([-cnt 1 1 X(n1)],Fbk1(n1),1);
            end
            cnt = cnt+1;
        end
    end
end

% LMI to enforce scalings to be non-negative
for i1 = 1:Niqc
    lmiterm([-(cnt) 1 1 lam(i1)],1,1);
    cnt = cnt+1;
end

%% Solve SDP: min gamsq subject to LMI constraints
lmisys = getlmis;
c = zeros(ndec,1);
c(end) = 1;

opt = [0 0 0 0 1];
[copt,xopt] = mincx(lmisys,c,opt);

%% Store solution
g = [];
info.P = [];
info.lamopt = [];
if ~isempty(xopt)
    g = sqrt(copt);
    for i1 = 1:nbasis
        info.P(:,:,i1) =  dec2mat(lmisys,xopt,X(i1));
    end
    lamopt = zeros(nM/2,nM/2,Niqc);
    Ropt = zeros(nM/2,nM/2,Niqc);
    Mopt = zeros(nM,nM,Niqc);
    for i = 1:Niqc
        lamopt(:,:,i) = dec2mat(lmisys,xopt,lam(i));
        Ropt(:,:,i) = dec2mat(lmisys,xopt,R(i));
        Mopt(:,:,i) = dec2mat(lmisys,xopt,Mi(i));
    end
    info.lamopt = lamopt;
    info.Ropt = Ropt;
    info.Mopt = Mopt;
end
